
//-------------
// Question 4 - Stratégie naturelle

// Jouons une fois
ta = 0.6; tb = 0.3;
n = 1000; // nb de fois que l'on joue
[g,tah,tbh,echec] = nat(ta,tb,n,1);

figure;
plot2d(1:n,g,1)
plot2d(1:n,tah,2)
plot2d(1:n,tbh,5)
title('Évolution des gains moyens')
legend(['global';'avec le bras A';'avec le bras B']);


// Si l'on répète l'experience, on voit que l'on choisit parfois le mauvais
// bras en définitive.

// tracer les suites de gains moyens pour plusieurs jeux
njeux = 32;  // (on a 32 couleurs de courbes possibles par défaut)
for i=1:njeux
    g = nat(ta,tb,n);
    plot2d(1:n,g,i);
end
// On voit que les gains moyens tendent parfois vers tA ou vers tb.
// Cela confirme qu'avec une proba>0, on finit par se tromper de bras.


// Estimons cette probabilité d'échec
// On va jouer un grand nombre de fois et on compte le nombre de fois
// que la procédure d'estimation échoue.

n = 100; //  nb de coups joués à chaque jeu
ns = 1000; // (un grand nombre...)
pe = 0;
for ind=1:ns
    [g,tah,tbh,echec] = nat(ta,tb,n,0);
    pe = pe+echec;
end
pe = pe/ns;
disp(['Probabilite d''échec estimée = ' string(pe)])
disp(['Valeur théorique = ' string(tb/(ta+tb))])


//-------------
// Questions 9 - 11 - Stratégie optimale

// Jouons une fois avec la stratégie optimale
ta = 0.6; tb = 0.3;
n = 10000; // nb de fois que l'on joue
[g,tah,tbh,echec] = opt(ta,tb,n,1);

figure;
plot2d(1:n,g,1)
plot2d(1:n,tah,2)
plot2d(1:n,tbh,5)
title('Évolution des gains moyens')
legend(['global';'avec le bras A';'avec le bras B']);

// Estimons la probabilité de choisir in fine le mauvais bras
// avec cette nouvelle stratégie
ta = 0.6; tb = 0.3;
n = 100; // nb de coups joués à chaque jeu
ns = 1000; // (un grand nombre...)
pe = 0;
for ind=1:ns
    [g,tah,tbh,echec] = opt(ta,tb,n,0);
    pe = pe+echec;
end
pe = pe/ns;
disp(['Probabilite d''échec estimée = ' string(pe)])

// Illustrons la gaussianité asymptotique du gain moyen
n = 10000;
ns = 10000;
gtab = zeros(1,ns);
for ind=1:ns
    [g,tahtab,tbhtab,echec] = opt(ta,tb,n,0);
    gtab(ind) = g($);
end

// illustration en superposant la FdR empirique avec la FdR limite
tm = max(ta,tb); s2 = tm*(1-tm);
D = sqrt(n)*(gtab-tm);
res = 1000;
xxb = 5*sqrt(s2);
xx = linspace(-xxb,xxb,res);
figure
plot2d2(-gsort(-D),(1:ns)/ns,rect=[-5 0 5 1])
plot2d(xx,cdfnor("PQ",xx,zeros(1,res),sqrt(s2)*ones(1,res)),5,rect=[-5 0 5 1])
legend(['FdR empirique','FdR limite']);

// illustration en superposant l'histogramme empirique et la densité limite
histplot(linspace(-xxb,xxb,50),D,1)
plot2d(xx,1/sqrt(s2*2*%pi)*exp( -xx.^2/(2*s2)),5)
// autre solution :
// histplot(500,D,rect=[-5 0 5 1])


//-------------
// Questions 12 - stratégie de Narendra

ta = 0.6; tb = 0.3;
n = 100000; // nb de fois que l'on joue
alpha = 1;

// Pour c = 1 <= 1/theta_b , c'est effectivement une bonne stratégie
[g,tah,tbh,echec,ptab] = narendra(ta,tb,n,alpha,1,1);

ptab($) // proba de jouer le bras A au dernier coup

figure;
plot2d(ptab)
title('Probabilité de jouer le bras A')

figure;
plot2d(1:n,g,1)
plot2d(1:n,tah,2)
plot2d(1:n,tbh,5)
title('Évolution des gains moyens')
legend(['global';'avec le bras A';'avec le bras B']);

// Pour c > 1/theta_b , ca n'est pas une bonne stratégie
[g,tah,tbh,echec,ptab] = narendra(ta,tb,n,alpha,2/tb,1);

ptab($) // proba de jouer le bras A au dernier coup

figure;
plot2d(ptab)
title('Probabilité de jouer le bras A')

figure;
plot2d(1:n,g,1)
plot2d(1:n,tah,2)
plot2d(1:n,tbh,5)
title('Évolution des gains moyens')
legend(['global';'avec le bras A';'avec le bras B']);
