
function [g,tahtab,tbhtab,echec] = nat(ta,tb,N,v)
    // ta : proba de gain avec le bras A
    // tb : proba de gain avec le bras B
    // N : nombre de fois que l'on joue
    // v : si v=1, affiche en console le bras préféré à la fin de la procédure.
    
    // règle la verbosité par défaut à 1
    if argn(2)<4 then
        v = 1;
    end
    
    tah = 0; gah = 0; na = 0;
    tbh = 0; gbh = 0; nb = 0;
    ge = zeros(1,N); // suite des gains élémentaires
    tahtab = zeros(1,N);
    tbhtab = zeros(1,N);
    for n=1:N
        if ( (tah>tbh)|((tah==tbh)&(rand()>0.5))) then
            // dans ce cas, on joue le levier A
            ge(n) = grand(1,1,"bin",1,ta);
            na = na + 1;
            // mise à jour des estimateurs
            tbh = tbh;
            gah = gah + ge(n); tah = 1/na*gah;
        else
            // sinon, on joue le levier B
            ge(n) = grand(1,1,"bin",1,tb);
            nb = nb + 1;
            // mise à jour des estimateurs
            tah = tah;
            gbh = gbh + ge(n); tbh = 1/nb*gbh;
        end
        // stocke les valeurs des estimateurs
        tahtab(n) = tah;
        tbhtab(n) = tbh;
    end
    // gain moyen jusqu'à l'étape n
    g = cumsum(ge)./(1:N);
    // affichage du bras préféré
    if v
        if tah>tbh then
            disp('A la fin on préfére le bras A')
        elseif tbh>tah then
            disp('A la fin on préfére le bras B')
        end
    end
    // est-ce que l'on s'est trompé de bras ?
    echec = (tbh<tah).*(ta<tb) + (tah<tbh).*(tb<ta); 
endfunction



function [g,tahtab,tbhtab,echec] = opt(ta,tb,N,v)
    // ta : proba de gain avec le bras A
    // tb : proba de gain avec le bras B
    // N : nombre de fois que l'on joue
    // v : si v=1, affiche en console le bras préféré à la fin de la procédure.
    
    // Pour la stratégie optimale, on décide de jouer A dès que
    // n est un cube parfait 
    // et de jouer B dès que
    // n-1 est un cube parfait.
    // (pour n=1 qui est ds les deux cas, on décide arbitrairement de jouer A)
    
    // règle la verbosité par défaut à 1
    if argn(2)<4 then
        v = 1;
    end
    
    // instants d'excitation des leviers A et B
    ea = zeros(1,N);
    eb = zeros(1,N);
    k = 1;
    while k^3<N
        ea(k^3) = 1; k=k+1;
    end
    k = 1;
    while k^3+1<N
        eb(k^3+1) = 1; k=k+1;
    end
    
    tah = 0; gah = 0; na = 0;
    tbh = 0; gbh = 0; nb = 0;
    ge = zeros(1,N); // suite des gains élémentaires
    tahtab = zeros(1,N);
    tbhtab = zeros(1,N);
    for n=1:N
        if ea(n) then
            // dans ce cas, on joue le levier A
            ge(n) = grand(1,1,"bin",1,ta);
            na = na + 1;
            // mise à jour des estimateurs
            tbh = tbh;
            gah = gah + ge(n); tah = 1/na*gah;
        elseif eb(n) then
            // dans ce cas, on joue le levier B
            ge(n) = grand(1,1,"bin",1,tb);
            nb = nb + 1;
            // mise à jour des estimateurs
            tah = tah;
            gbh = gbh + ge(n); tbh = 1/nb*gbh;                
        elseif tah>tbh then
            // dans ce cas, on joue le levier A
            ge(n) = grand(1,1,"bin",1,ta);
            na = na + 1;
            // mise à jour des estimateurs
            tbh = tbh;
            gah = gah + ge(n); tah = 1/na*gah;
        else
            // sinon, on joue le levier B
            ge(n) = grand(1,1,"bin",1,tb);
            nb = nb + 1;
            // mise à jour des estimateurs
            tah = tah;
            gbh = gbh + ge(n); tbh = 1/nb*gbh;
        end
        // stocke les valeurs des estimateurs
        tahtab(n) = tah;
        tbhtab(n) = tbh;
    end
    // gain moyen jusqu'à l'étape n
    g = cumsum(ge)./(1:N);
    // affichage du bras préféré
    if v
        if tah>tbh then
            disp('A la fin on préfére le bras A')
        elseif tbh>tah then
            disp('A la fin on préfére le bras B')
        end
    end
    // est-ce que l'on s'est trompé de bras ?
    echec = (tbh<tah).*(ta<tb) + (tah<tbh).*(tb<ta); 
endfunction



function g = opt_speed(ta,tb,N)
    // ta : proba de gain avec le bras A
    // tb : proba de gain avec le bras B
    // N : nombre de fois que l'on joue

    // version rapide du bandit avec stratégie optimale
    // (ne conserve pas les résultats intermédiaires)
 
    // instants d'excitation des leviers A et B
    ea = zeros(1,N);
    eb = zeros(1,N);
    k = 1;
    while k^3<N
        ea(k^3) = 1; k=k+1;
    end
    k = 1;
    while k^3+1<N
        eb(k^3+1) = 1; k=k+1;
    end
 
    tah = 0; gah = 0; na = 0;
    tbh = 0; gbh = 0; nb = 0;
    for n=1:N
        if ea(n) then
            // dans ce cas, on joue le levier A
            ge = grand(1,1,"bin",1,ta);
            na = na + 1;
            // mise à jour des estimateurs
            tbh = tbh;
            gah = gah + ge; tah = 1/na*gah;
        elseif eb(n) then
            // dans ce cas, on joue le levier B
            ge = grand(1,1,"bin",1,tb);
            nb = nb + 1;
            // mise à jour des estimateurs
            tah = tah;
            gbh = gbh + ge; tbh = 1/nb*gbh;                
        elseif tah>tbh then
            // dans ce cas, on joue le levier A
            ge = grand(1,1,"bin",1,ta);
            na = na + 1;
            // mise à jour des estimateurs
            tbh = tbh;
            gah = gah + ge; tah = 1/na*gah;
        else
            // sinon, on joue le levier B
            ge = grand(1,1,"bin",1,tb);
            nb = nb + 1;
            // mise à jour des estimateurs
            tah = tah;
            gbh = gbh + ge; tbh = 1/nb*gbh;
        end
    end
    // gain moyen final
    g = (gah+gbh)/N;
endfunction


function [g,tahtab,tbhtab,echec,ptab] = narendra(ta,tb,N,alpha,c,v)
    // ta : proba de gain avec le bras A
    // tb : proba de gain avec le bras B
    // N : nombre de fois que l'on joue
    //alpha, c : constante définissant les gamma_n = (c / (n+c) )^alpha
    // v : si v=1, affiche en console le bras préféré à la fin de la procédure.
    
    // règle la verbosité par défaut à 1
    // NB : on borne gamma_n par 0.1 car s'il est trop proche de 1 au départ
    // il peut tomber rapidement dans le mauvais bras et à cause de la
    // précision numérique limitée y restera coincé indéfiniment.
    
    if argn(2)<5 then
        v = 1;
    end
    
    tah = 0; gah = 0; na = 0;
    tbh = 0; gbh = 0; nb = 0;
    ge = zeros(1,N); // suite des gains élémentaires
    p = 1/2; // proba de décision
    ptab = zeros(1,N); // les stocker
    tahtab = zeros(1,N);
    tbhtab = zeros(1,N);
    for n=1:N
        if (n>1) then
            if ge(n-1)>0 then
                // on met à jour si l'on a gagné à l'étape précédente
                gam = c/(n-1+c); // valeur de gamma_(n-1)
                // gam = c/(n-1+c); // valeur de gamma_(n-1)
                // gam = 1/n;
                p = p + L*gam*(1-p) - (1-L)*gam*p;
            end
            ptab(n) = p;
        end
        L = grand(1,1,"bin",1,p);
        if L then
            // dans ce cas, on joue le levier A
            ge(n) = grand(1,1,"bin",1,ta);
            na = na + 1;
            // mise à jour des estimateurs
            tbh = tbh;
            gah = gah + ge(n); tah = 1/na*gah;
        else
            // sinon, on joue le levier B
            ge(n) = grand(1,1,"bin",1,tb);
            nb = nb + 1;
            // mise à jour des estimateurs
            tah = tah;
            gbh = gbh + ge(n); tbh = 1/nb*gbh;
        end
        // stocke les valeurs des estimateurs
        tahtab(n) = tah;
        tbhtab(n) = tbh;
    end
    // gain moyen jusqu'à l'étape n
    g = cumsum(ge)./(1:N);
    // affichage du bras préféré
    if v
        if tah>tbh then
            disp('A la fin on préfére le bras A')
        elseif tbh>tah then
            disp('A la fin on préfére le bras B')
        end
    end
    // est-ce que l'on s'est trompé de bras ?
    echec = (tbh<tah).*(ta<tb) + (tah<tbh).*(tb<ta); 
endfunction

