
function [X,S,d] = jeu(a,b,p)
    // a : avoir de départ
    // b : gain souhaité pour arrêt du jeu
    // p : probabilité de gain à chaque étape
    // X : résultats des épreuves élémentaires
    // S : avoir au cours du temps
    // d : durée de la partie (d = length(S)-1)
    
    if (b<=a) then
        disp('bornes dans le mauvais sens')
    end
    
    d = 0;
    X(1) = 0;
    S(1) = a;
    while ((S($)>0)&(S($)<b))
        acc = -1+2*grand(1,1,"bin",1,p);
        X = [X acc];
        S = [S S($)+acc];
        d = d+1;
    end
endfunction


function [ruine,d] = jeu_speed(a,b,p)
    // variante de jeu où l'on ne garde pas les résultats intermédiaires
    // a : avoir de départ
    // b : gain souhaité pour arrêt du jeu
    // p : probabilité de gain à chaque étape
    // ruine : résultat final (ruine=T si le jour est ruiné)
    // d : durée de la partie (d = length(S)-1)
    
    if (b<=a) then
        disp('bornes dans le mauvais sens')
    end
    
    d = 0;
    s = a;
    while ((s>0)&(s<b))
        acc = -1+2*grand(1,1,"bin",1,p);
        s = s+acc;
        d = d+1;
    end
    ruine = (s==0);
endfunction


function histo_durees(a,b,p,ns)
    // ns : nb d'échantillons
    dtab = zeros(1,ns);
    for ind=1:ns
        [ruine,d] = jeu_speed(a,b,p);
        dtab(ind) = d;
    end

    dmax = max(dtab);
    h = zeros(1,dmax);
    for d=1:dmax
        h(d) = sum(dtab==d);
    end
    h = h/ns;
    plot2d3(1:dmax,h)
    xtitle('Histogramme empirique des durées')
endfunction


function [dme,pre] = estim_dureemoy_pruine(a,b,p,ns)
    // ns = nb d'échantillons
    // dme = durée moyenne estimée
    // pre = probabilité de ruine estimée
    dme = 0;
    pre = 0;
    for n=1:ns
        [ruine,d] = jeu_speed(a,b,p);
        dme = dme+d;
        pre =  pre+ruine;
    end
    dme = dme/ns;
    pre = pre/ns;
endfunction


function [dm,pr] = dureemoy_pruine(a,b,p)
    // dme = durée moyenne théorique
    // pre = probabilité de ruine théorique
    // Attention : cette fonction ne gere pas le cas de la valeur p=0.5
    q = 1-p;
    pr = ( (q./p).^a - (q./p).^b )./( 1 - (q./p).^b );
    dm = 1 ./(p-q).*(b*(1-pr)-a);
endfunction
